@@ -9,6 +9,7 @@ import Control.Applicative ((<$>))
 import Debian.Debianize.Optparse(_flags, parseProgramArguments)
 import Control.Lens
 import Data.Algorithm.DiffContext (getContextDiff, prettyContextDiff)
+import Data.Bool (bool)
 import Data.Function (on)
 import Data.List (sortBy)
 import Data.Map as Map (differenceWithKey, insert, intersectionWithKey)
@@ -37,11 +38,12 @@ import Debian.Pretty (ppShow)
 import Debian.Policy (databaseDirectory, PackageArchitectures(All), PackagePriority(Extra), parseMaintainer, Section(MainSection), SourceFormat(Native3), StandardsVersion(..), getDebhelperCompatLevel, getDebianStandardsVersion, License(..))
 import Debian.Relation (BinPkgName(..), Relation(..), SrcPkgName(..), VersionReq(..))
 import Debian.Release (ReleaseName(ReleaseName, relName))
-import Debian.Version (parseDebianVersion, buildDebianVersion)
+import Debian.Version (parseDebianVersion', buildDebianVersion)
 import Distribution.Compiler (CompilerFlavor(GHC))
 import Distribution.Package (PackageName(PackageName))
 import Prelude hiding (log)
-import System.Exit (ExitCode(ExitSuccess))
+import System.Directory (doesDirectoryExist)
+import System.Exit (exitWith, ExitCode(..))
 import System.FilePath ((</>))
 import System.Process (readProcessWithExitCode)
 import Test.HUnit
@@ -181,7 +183,7 @@ test2 label =
                               finalizeDebianization)
                           atoms
                  diff <- diffDebianizations (view debInfo (expect atoms)) (view debInfo deb)
-                 assertEqual label [] diff)
+                 assertEmptyDiff label diff)
     where
       expect atoms =
           execCabalM
@@ -234,7 +236,7 @@ test3 label =
               do atoms <- testAtoms
                  deb <- (execCabalT (liftCabal inputDebianization) atoms)
                  diff <- diffDebianizations (view debInfo (testDeb2 atoms)) (view debInfo deb)
-                 assertEqual label [] diff)
+                 assertEmptyDiff label diff)
     where
       testDeb2 :: CabalInfo -> CabalInfo
       testDeb2 atoms =
@@ -294,7 +296,7 @@ test3 label =
                 (debInfo . D.control . S.uploaders) .= [NameAddr {nameAddr_name = Just "Marco Silva", nameAddr_addr = "marcot@debian.org"},NameAddr {nameAddr_name = Just "Joachim Breitner", nameAddr_addr = "nomeata@debian.org"}]
                 (debInfo . D.control . S.priority) .= Just Extra
                 (debInfo . D.control . S.section) .= Just (MainSection "haskell")
-                (debInfo . D.control . S.buildDepends) %= (++ [[Rel (BinPkgName {unBinPkgName = "debhelper"}) (Just (GRE (Debian.Version.parseDebianVersion ("7" :: String)))) Nothing]])
+                (debInfo . D.control . S.buildDepends) %= (++ [[Rel (BinPkgName {unBinPkgName = "debhelper"}) (Just (GRE (Debian.Version.parseDebianVersion' ("7" :: String)))) Nothing]])
                 (debInfo . D.control . S.buildDependsIndep) %=  (++ [[Rel (BinPkgName {unBinPkgName = "perl"}) Nothing Nothing]])
                 (debInfo . D.control . S.standardsVersion) .= Just (StandardsVersion 3 9 4 Nothing)
                 (debInfo . D.control . S.vcsFields) %= Set.union (Set.fromList [ S.VCSBrowser "http://darcs.debian.org/cgi-bin/darcsweb.cgi?r=pkg-haskell/haskell-devscripts"
@@ -316,19 +318,19 @@ test3 label =
                      [ [Rel (BinPkgName {unBinPkgName = "dctrl-tools"}) Nothing Nothing]
                      , [Rel (BinPkgName {unBinPkgName = "debhelper"}) Nothing Nothing]
                      , [Rel (BinPkgName {unBinPkgName = "dh-buildinfo"}) Nothing Nothing]
-                     , [Rel (BinPkgName {unBinPkgName = "ghc"}) (Just (GRE (Debian.Version.parseDebianVersion ("7.6" :: String)))) Nothing]
+                     , [Rel (BinPkgName {unBinPkgName = "ghc"}) (Just (GRE (Debian.Version.parseDebianVersion'("7.6" :: String)))) Nothing]
                      , [Rel (BinPkgName {unBinPkgName = "cdbs"}) Nothing Nothing]
                      , [Rel (BinPkgName {unBinPkgName = "${misc:Depends}"}) Nothing Nothing]
                      , [Rel (BinPkgName {unBinPkgName = "html-xml-utils"}) Nothing Nothing]
-                     , [Rel (BinPkgName {unBinPkgName = "hscolour"}) (Just (GRE (Debian.Version.parseDebianVersion ("1.8" :: String)))) Nothing]
-                     , [Rel (BinPkgName {unBinPkgName = "ghc-haddock"}) (Just (GRE (Debian.Version.parseDebianVersion ("7.4" :: String)))) Nothing] ]
+                     , [Rel (BinPkgName {unBinPkgName = "hscolour"}) (Just (GRE (Debian.Version.parseDebianVersion'("1.8" :: String)))) Nothing]
+                     , [Rel (BinPkgName {unBinPkgName = "ghc-haddock"}) (Just (GRE (Debian.Version.parseDebianVersion'("7.4" :: String)))) Nothing] ]
 {-
                 control %= (\ y -> y { S.source = 
                                      , S.maintainer = Just (NameAddr {nameAddr_name = Just "Debian Haskell Group", nameAddr_addr = "pkg-haskell-maintainers@lists.alioth.debian.org"})
                                      , S.uploaders = [NameAddr {nameAddr_name = Just "Marco Silva", nameAddr_addr = "marcot@debian.org"},NameAddr {nameAddr_name = Just "Joachim Breitner", nameAddr_addr = "nomeata@debian.org"}]
                                      , S.priority = Just Extra
                                      , S.section = Just (MainSection "haskell")
-                                     , S.buildDepends = (S.buildDepends y) ++ [[Rel (BinPkgName {unBinPkgName = "debhelper"}) (Just (GRE (Debian.Version.parseDebianVersion ("7" :: String)))) Nothing]]
+                                     , S.buildDepends = (S.buildDepends y) ++ [[Rel (BinPkgName {unBinPkgName = "debhelper"}) (Just (GRE (Debian.Version.parseDebianVersion'("7" :: String)))) Nothing]]
                                      , S.buildDependsIndep = (S.buildDependsIndep y) ++ [[Rel (BinPkgName {unBinPkgName = "perl"}) Nothing Nothing]]
                                      , S.standardsVersion = Just (StandardsVersion 3 9 4 Nothing)
                                      , S.vcsFields = Set.union (S.vcsFields y) (Set.fromList [ S.VCSBrowser "http://darcs.debian.org/cgi-bin/darcsweb.cgi?r=pkg-haskell/haskell-devscripts"
@@ -356,12 +358,12 @@ test3 label =
                                                                                               [ [Rel (BinPkgName {unBinPkgName = "dctrl-tools"}) Nothing Nothing]
                                                                                               , [Rel (BinPkgName {unBinPkgName = "debhelper"}) Nothing Nothing]
                                                                                               , [Rel (BinPkgName {unBinPkgName = "dh-buildinfo"}) Nothing Nothing]
-                                                                                              , [Rel (BinPkgName {unBinPkgName = "ghc"}) (Just (GRE (Debian.Version.parseDebianVersion ("7.6" :: String)))) Nothing]
+                                                                                              , [Rel (BinPkgName {unBinPkgName = "ghc"}) (Just (GRE (Debian.Version.parseDebianVersion'("7.6" :: String)))) Nothing]
                                                                                               , [Rel (BinPkgName {unBinPkgName = "cdbs"}) Nothing Nothing]
                                                                                               , [Rel (BinPkgName {unBinPkgName = "${misc:Depends}"}) Nothing Nothing]
                                                                                               , [Rel (BinPkgName {unBinPkgName = "html-xml-utils"}) Nothing Nothing]
-                                                                                              , [Rel (BinPkgName {unBinPkgName = "hscolour"}) (Just (GRE (Debian.Version.parseDebianVersion ("1.8" :: String)))) Nothing]
-                                                                                              , [Rel (BinPkgName {unBinPkgName = "ghc-haddock"}) (Just (GRE (Debian.Version.parseDebianVersion ("7.4" :: String)))) Nothing] ]
+                                                                                              , [Rel (BinPkgName {unBinPkgName = "hscolour"}) (Just (GRE (Debian.Version.parseDebianVersion'("1.8" :: String)))) Nothing]
+                                                                                              , [Rel (BinPkgName {unBinPkgName = "ghc-haddock"}) (Just (GRE (Debian.Version.parseDebianVersion'("7.4" :: String)))) Nothing] ]
                                                                                             , B.recommends = []
                                                                                             , B.suggests = []
                                                                                             , B.preDepends = []
@@ -374,14 +376,14 @@ test3 label =
                                                                                             )
             atoms
       log = ChangeLog [Entry { logPackage = "haskell-devscripts"
-                             , logVersion = Debian.Version.parseDebianVersion ("0.8.13" :: String)
+                             , logVersion = Debian.Version.parseDebianVersion'("0.8.13" :: String)
                              , logDists = [ReleaseName {relName = "experimental"}]
                              , logUrgency = "low"
                              , logComments = "  [ Joachim Breitner ]\n  * Improve parsing of \"Setup register\" output, patch by David Fox\n  * Enable creation of hoogle files, thanks to Kiwamu Okabe for the\n    suggestion. \n\n  [ Kiwamu Okabe ]\n  * Need --html option to fix bug that --hoogle option don't output html file.\n  * Support to create /usr/lib/ghc-doc/hoogle/*.txt for hoogle package.\n\n  [ Joachim Breitner ]\n  * Symlink hoogle\8217s txt files to /usr/lib/ghc-doc/hoogle/\n  * Bump ghc dependency to 7.6 \n  * Bump standards version\n"
                              , logWho = "Joachim Breitner <nomeata@debian.org>"
                              , logDate = "Mon, 08 Oct 2012 21:14:50 +0200" },
                        Entry { logPackage = "haskell-devscripts"
-                             , logVersion = Debian.Version.parseDebianVersion ("0.8.12" :: String)
+                             , logVersion = Debian.Version.parseDebianVersion'("0.8.12" :: String)
                              , logDists = [ReleaseName {relName = "unstable"}]
                              , logUrgency = "low"
                              , logComments = "  * Depend on ghc >= 7.4, adjusting to its haddock --interface-version\n    behaviour.\n"
@@ -400,7 +402,7 @@ test4 label =
                  new <- withCurrentDirectory inTop $ do
                           execCabalT (debianize (defaultAtoms >> customize log)) atoms
                  diff <- diffDebianizations (view debInfo old) (view debInfo ({-copyFirstLogEntry old-} new))
-                 assertEqual label [] diff)
+                 assertEmptyDiff label diff)
     where
       customize :: Maybe ChangeLog -> CabalT IO ()
       customize log =
@@ -507,7 +509,7 @@ test5 label =
                      level = view D.compat old
                  new <- withCurrentDirectory inTop (execCabalT (debianize (defaultAtoms >> customize old level standards)) atoms)
                  diff <- diffDebianizations old (view debInfo new)
-                 assertEqual label [] diff)
+                 assertEmptyDiff label diff)
     where
       customize old level standards =
           do (A.debInfo . D.utilsPackageNameBase) .= Just "creativeprompts-data"
@@ -582,17 +584,27 @@ test5 label =
       jstreePath = "/usr/share/clckwrks-0.13.2/jstree"
       json2Path = "/usr/share/clckwrks-0.13.2/json2"
 
+-- | Obviously this is a hack, but the builddir is set to "dist-ghc"
+-- by haskell-devscripts, while by default cabal sets it to "dist".
+findBuildDir :: IO FilePath
+findBuildDir = doesDirectoryExist "dist-ghc" >>= return . bool "dist" "dist-ghc"
+
 test6 :: String -> Test
 test6 label =
     TestLabel label $
-    TestCase (do result <- readProcessWithExitCode "runhaskell" ["--ghc-arg=-package-db=dist/package.conf.inplace", "test-data/artvaluereport2/input/debian/Debianize.hs", "--dry-run"] ""
+    TestCase (do dist <- findBuildDir
+                 result <- readProcessWithExitCode "runhaskell" ["--ghc-arg=-package-db=" ++ dist ++ "/package.conf.inplace", "test-data/artvaluereport2/input/debian/Debianize.hs", "--dry-run"] ""
                  assertEqual label (ExitSuccess, "", "") result)
 
 test7 :: String -> Test
 test7 label =
     TestLabel label $
-    TestCase (do new <- readProcessWithExitCode "runhaskell" ["--ghc-arg=-package-db=dist/package.conf.inplace", "debian/Debianize.hs", "--dry-run", "--native"] ""
-                 assertEqual label (ExitSuccess, "Ignored debianization file: debian/cabal-debian.1\nIgnored debianization file: debian/cabal-debian.manpages\nDebianization (dry run):\n  No changes\n\n", "") new)
+    TestCase (do dist <- findBuildDir
+                 result <- readProcessWithExitCode "runhaskell" ["--ghc-arg=-package-db=" ++ dist ++ "/package.conf.inplace", "debian/Debianize.hs", "--dry-run", "--native"] ""
+                 assertEqual label (ExitSuccess, "Ignored debianization file: debian/cabal-debian.1\nIgnored debianization file: debian/cabal-debian.manpages\nDebianization (dry run):\n  No changes\n\n", "") result
+{-
+                 assertBool label (elem new [(ExitSuccess, "Ignored debianization file: debian/cabal-debian.1\nIgnored debianization file: debian/cabal-debian.manpages\nDebianization (dry run):\n  No changes\n\n", ""),
+                                             (ExitSuccess, "Ignored debianization file: debian/cabal-debian.manpages\nIgnored debianization file: debian/cabal-debian.1\nDebianization (dry run):\n  No changes\n\n", "")]) -})
 
 test8 :: String -> Test
 test8 label =
@@ -603,7 +615,7 @@ test8 label =
                   let log = view D.changelog old
                   new <- withCurrentDirectory inTop $ newFlags >>= newCabalInfo >>= execCabalT (debianize (defaultAtoms >> customize log))
                   diff <- diffDebianizations old (view debInfo new)
-                  assertEqual label [] diff
+                  assertEmptyDiff label diff
              )
     where
       customize Nothing = error "Missing changelog"
@@ -623,7 +635,7 @@ test9 label =
                  let Just (ChangeLog (entry : _)) = view (debInfo . D.changelog) new
                  old <- withCurrentDirectory outTop $ newFlags >>= execDebianT (inputDebianization >> copyChangelogDate (logDate entry)) . D.makeDebInfo
                  diff <- diffDebianizations old (view debInfo new)
-                 assertEqual label [] diff)
+                 assertEmptyDiff label diff)
     where
       customize =
           do newDebianization' (Just 9) (Just (StandardsVersion 3 9 6 Nothing))
@@ -644,7 +656,7 @@ test9 label =
                    , "AlexWrapper-strict-bytestring"]
              (debInfo . D.control . S.homepage) .= Just "http://www.haskell.org/alex/"
              (debInfo . D.sourceFormat) .= Native3
-             (debInfo . D.debVersion) .= Just (parseDebianVersion ("3.0.2-1~hackage1" :: String))
+             (debInfo . D.debVersion) .= Just (parseDebianVersion'("3.0.2-1~hackage1" :: String))
              doExecutable (BinPkgName "alex")
                           (D.InstallFile {D.execName = "alex", D.destName = "alex", D.sourceDir = Nothing, D.destDir = Nothing})
              -- Bootstrap self-dependency
@@ -660,7 +672,7 @@ test10 label =
                  let Just (ChangeLog (entry : _)) = view D.changelog old
                  new <- withCurrentDirectory inTop $ newFlags >>= newCabalInfo >>= execCabalT (debianize (defaultAtoms >> customize >> (liftCabal $ copyChangelogDate $ logDate entry)))
                  diff <- diffDebianizations old (view debInfo new)
-                 assertEqual label [] diff)
+                 assertEmptyDiff label diff)
     where
       customize :: CabalT IO ()
       customize =
@@ -721,5 +733,19 @@ sortBinaryDebs :: DebianT IO ()
 sortBinaryDebs = (D.control . S.binaryPackages) %= sortBy (compare `on` view B.package)
 
 main :: IO ()
-main = runTestTT tests >>= putStrLn . show
+main = do
+ counts <- runTestTT tests
+ exitWith $ if errors counts + failures counts > 0
+            then ExitFailure 1
+            else ExitSuccess
 
+
+-- Cusstom HUnit assertion, which prints the diffs without using 'show'
+assertEmptyDiff :: String -- ^ The message prefix
+                -> String -- ^ The actual diff
+                -> Assertion
+assertEmptyDiff preface "" = return ()
+assertEmptyDiff preface diff = assertFailure $
+ (if null preface then "" else preface ++ "\n") ++
+ "diff not empty:\n" ++
+ diff
