@@ -44,7 +44,7 @@ import Debian.Relation (BinPkgName, BinPkgName(BinPkgName), Relation(Rel), Relat
 import qualified Debian.Relation as D (BinPkgName(BinPkgName), Relation(..))
 import Debian.Release (parseReleaseName)
 import Debian.Time (getCurrentLocalRFC822Time)
-import qualified Debian.Version as V (buildDebianVersion, DebianVersion, parseDebianVersion, epoch, version, revision)
+import qualified Debian.Version as V (buildDebianVersion, DebianVersion, parseDebianVersion', epoch, version, revision)
 import Distribution.Compiler (CompilerFlavor(GHC))
 #if MIN_VERSION_Cabal(1,22,0)
 import Distribution.Compiler (CompilerFlavor(GHCJS))
@@ -53,6 +53,7 @@ import Distribution.Package (Dependency(..), PackageIdentifier(..), PackageName(
 import Distribution.PackageDescription as Cabal (allBuildInfo, author, BuildInfo(buildable, extraLibs), Executable(buildInfo, exeName), FlagName(FlagName), maintainer, PackageDescription(testSuites))
 import qualified Distribution.PackageDescription as Cabal (PackageDescription(dataFiles, executables, library, package))
 import Prelude hiding (init, log, map, unlines, unlines, writeFile)
+import System.Directory (doesFileExist)
 import System.FilePath ((<.>), (</>), makeRelative, splitFileName, takeDirectory, takeFileName)
 import System.IO (hPutStrLn, stderr)
 import Text.ParserCombinators.Parsec.Rfc2822 (NameAddr(..))
@@ -75,7 +76,8 @@ finalizeDebianization =
     do date <- liftIO getCurrentLocalRFC822Time
        currentUser <- liftIO getCurrentDebianUser
        debhelperCompat <- liftIO getDebhelperCompatLevel
-       finalizeDebianization' date currentUser debhelperCompat
+       setupExists <- or <$> mapM (liftIO . doesFileExist) ["Setup.hs", "Setup.lhs"]
+       finalizeDebianization' date currentUser debhelperCompat setupExists
        vb <- use (A.debInfo . D.flags . verbosity)
        when (vb >= 3) (get >>= \ x -> liftIO (putStrLn ("\nFinalized Cabal Info: " ++ show x ++ "\n")))
        either (\e -> liftIO $ hPutStrLn stderr ("WARNING: " ++ e)) (\_ -> return ()) =<< use (A.debInfo . D.control . S.maintainer)
@@ -88,8 +90,8 @@ finalizeDebianization =
 -- this function is not idempotent.  (Exported for use in unit tests.)
 -- FIXME: we should be able to run this without a PackageDescription, change
 --        paramter type to Maybe PackageDescription and propagate down thru code
-finalizeDebianization'  :: (Monad m, Functor m) => String -> Maybe NameAddr -> Maybe Int -> CabalT m ()
-finalizeDebianization' date currentUser debhelperCompat =
+finalizeDebianization'  :: (Monad m, Functor m) => String -> Maybe NameAddr -> Maybe Int -> Bool -> CabalT m ()
+finalizeDebianization' date currentUser debhelperCompat setupExists =
     do -- In reality, hcs must be a singleton or many things won't work.  But some day...
        hc <- use (A.debInfo . D.flags . compilerFlavor)
        pkgDesc <- use A.packageDescription
@@ -100,6 +102,9 @@ finalizeDebianization' date currentUser debhelperCompat =
          (True, D.TestsRun) -> (A.debInfo . D.rulesSettings) %= (++ ["DEB_ENABLE_TESTS = yes"])
          (True, D.TestsBuild) -> (A.debInfo . D.rulesSettings) %= (++ ["DEB_ENABLE_TESTS = yes", "DEB_BUILD_OPTIONS += nocheck"])
          _ -> return ()
+       case setupExists of
+         False -> (A.debInfo . D.rulesSettings) %= (++ ["DEB_SETUP_BIN_NAME = cabal"])
+         _ -> return ()
 
        finalizeSourceName B.HaskellSource
        checkOfficialSettings hc
@@ -157,7 +162,7 @@ finalizeDescription bdd =
 debianVersion :: (Monad m, Functor m) => CabalT m V.DebianVersion
 debianVersion =
     do cabalName <- (pkgName . Cabal.package) <$> use A.packageDescription
-       (cabalVersion :: V.DebianVersion) <- (V.parseDebianVersion . ppShow . pkgVersion . Cabal.package) <$> use A.packageDescription
+       (cabalVersion :: V.DebianVersion) <- (V.parseDebianVersion' . ppShow . pkgVersion . Cabal.package) <$> use A.packageDescription
        cabalEpoch <- debianEpoch cabalName
        fmt <- use (A.debInfo . D.sourceFormat)
        cabalRevision <-
@@ -182,12 +187,12 @@ debianVersion =
          _ | isJust versionArg -> return $ fromJust versionArg
          _ | isJust debVersion ->
                case (V.epoch (fromJust debVersion),
-                     V.parseDebianVersion (V.version (fromJust debVersion)),
+                     V.parseDebianVersion' (V.version (fromJust debVersion)),
                      V.revision (fromJust debVersion)) of
                  (debEpoch, debianVersion', (debianRevision :: Maybe String)) ->
                      let finalEpoch = max debEpoch cabalEpoch
                          finalVersion = max debianVersion' cabalVersion
-                         (finalRevision :: Maybe String) = maximumBy (compare `on` fmap V.parseDebianVersion) [debianRevision, cabalRevision] in
+                         (finalRevision :: Maybe String) = maximumBy (compare `on` fmap V.parseDebianVersion') [debianRevision, cabalRevision] in
                      return $ V.buildDebianVersion finalEpoch (ppShow finalVersion) finalRevision
          _ -> return $ V.buildDebianVersion cabalEpoch (ppShow cabalVersion) cabalRevision
 
@@ -346,19 +351,23 @@ checkOfficialSettings flavor =
                   _ -> error $ "There is no official packaging for " ++ show flavor
 
 officialSettings :: (Monad m, Functor m) => CabalT m ()
-officialSettings =
-    do pkgDesc <- use A.packageDescription
-       let PackageName cabal = pkgName (Cabal.package pkgDesc)
+officialSettings = do
+    pkgDesc <- use A.packageDescription
+    let PackageName cabal = pkgName (Cabal.package pkgDesc)
+    zoom A.debInfo $ do
+        let officialError = error "officialSettings: no sourcePackageName"
 
-       (A.debInfo . D.control . S.standardsVersion) .?= Just (parseStandardsVersion "3.9.6")
-       (A.debInfo . D.control . S.homepage) .?= Just ("http://hackage.haskell.org/package/" <> pack cabal)
-       (A.debInfo . D.omitProfVersionDeps) .= True
-       SrcPkgName src <- use (A.debInfo . D.sourcePackageName) >>= maybe (error "officialSettings: no sourcePackageName") return
+        D.omitProfVersionDeps .= True
+        SrcPkgName src <- fromMaybe officialError <$> use D.sourcePackageName
 
-       (A.debInfo . D.control . S.vcsFields) %= Set.union (Set.fromList
-          [ S.VCSBrowser $ "http://darcs.debian.org/cgi-bin/darcsweb.cgi?r=pkg-haskell/" <> pack src
-          , S.VCSDarcs  $ "http://darcs.debian.org/pkg-haskell/" <> pack src
-          ])
+        let packagesURI = "https://anonscm.debian.org/cgit/pkg-haskell/DHG_packages.git/tree/p/" <> pack src
+        zoom D.control $ do
+           S.standardsVersion .?= Just (parseStandardsVersion "3.9.6")
+           S.homepage .?= Just ("http://hackage.haskell.org/package/" <> pack cabal)
+           S.vcsFields %= Set.union (Set.fromList
+              [ S.VCSBrowser packagesURI
+              , S.VCSGit  "git://git.debian.org/git/pkg-haskell/DHG_packages.git"
+              ])
 
 putBuildDeps :: (Monad m, Functor m) => (Relations -> Relations) -> PackageDescription -> CabalT m ()
 putBuildDeps finalizeRelations pkgDesc =
